@@ -34,15 +34,37 @@ class PysolRandom(RandomBase):
 # ************************************************************************/
 
 
-class LCRandom64(PysolRandom):
+class _LCBase(PysolRandom):
+    def __init__(self, seed=None):
+        if seed is None:
+            seed = self._getRandomSeed()
+        PysolRandom.__init__(self, seed)
+        self.initial_seed = seed
+        self.setSeed(seed)
+        self.origin = self.ORIGIN_UNKNOWN
+
+    def reset(self):
+        self.setSeed(self.initial_seed)
+
+    def getSeedAsStr(self):
+        return self._getSeedStrPrefix() + str(self.seed)
+
+    def getSeedStr(self):
+        return self._getSeedStrPrefix() + str(self.initial_seed)
+
+
+class LCRandom64(_LCBase):
     MAX_SEED = 0xffffffffffffffff  # 64 bits
 
     def random(self):
         self.seed = (self.seed * 6364136223846793005 + 1) & self.MAX_SEED
         return ((self.seed >> 21) & 0x7fffffff) / 2147483648.0
 
+    def _getSeedStrPrefix(self):
+        return "old"
 
-class LCRandom31(RandomBase):
+
+class LCRandom31(_LCBase):
     MAX_SEED = ((1 << (31 + 2)) - 1)         # 33 bits
 
     def setSeed(self, seed):
@@ -50,6 +72,7 @@ class LCRandom31(RandomBase):
             raise ValueError("seed is out of range")
         self.seed = seed
         self.seedx = seed if (seed < 0x100000000) else (seed - 0x100000000)
+        return self.seed
 
     def random(self):
         if (self.seed < 0x100000000):
@@ -69,6 +92,19 @@ class LCRandom31(RandomBase):
     def randint(self, a, b):
         return a + self.random() % (b + 1 - a)
 
+    def _getSeedStrPrefix(self):
+        return "ms"
+
+    def increaseSeed(self, seed):
+        """docstring for increaseSeed"""
+        integer_seed = super(LCRandom31, self).increaseSeed(seed)
+        return "ms{}".format(integer_seed)
+
+    def str(self, seed):
+        if match_ms_deal_prefix(seed) is None:
+            return "%05d" % int(seed)
+        return seed
+
 
 # * Mersenne Twister random number generator
 class MTRandom(RandomBase, random2.Random):
@@ -86,9 +122,10 @@ class MTRandom(RandomBase, random2.Random):
     def setSeed(self, seed):
         random2.Random.__init__(self, seed)
         self.initial_state = self.getstate()
+        return seed
 
     def reset(self):
-        self.setstate(self.initial_state)
+        self.setSeed(self.initial_seed)
 
 
 def shuffle(cards, game_num, which_deals):
@@ -105,3 +142,26 @@ _ms_pat = re.compile("ms([0-9]+)\\n?\\Z")
 def match_ms_deal_prefix(mystring):
     ret = re.match(_ms_pat, mystring)
     return None if not ret else int(ret.group(1))
+
+
+MS_LONG_BIT = (1 << 1000)
+CUSTOM_BIT = (1 << 999)
+
+
+def random__str2int(s):
+    if s == 'Custom':
+        return CUSTOM_BIT | MS_LONG_BIT
+    m = match_ms_deal_prefix(s)
+    if m is not None:
+        return (m | MS_LONG_BIT)
+    else:
+        return int(s)
+
+
+def random__int2str(l):
+    if ((l & MS_LONG_BIT) != 0):
+        if ((l & CUSTOM_BIT) != 0):
+            return 'Custom'
+        return "ms" + str(l & (~ MS_LONG_BIT))
+    else:
+        return str(l)
